import { Logger } from '../Logger'
import { getBaseOption, BaseOption } from '../hs-cti/storage'
import type { CTIRes } from '../hs-cti/outputType'

export interface Request {
  baseUrl: string
  url: string
  data: object
}

const apiLogger = new Logger(
  window.ctiLoggerLevel || getBaseOption(BaseOption.LoggerLevel),
  'HsApi'
)

export const random16Hex = () =>
  (0x10000 | (Math.random() * 0x10000)).toString(16).substr(1)
export const random64Hex = () =>
  random16Hex() + random16Hex() + random16Hex() + random16Hex()

export async function JPOST({ baseUrl, url, data }: Request): Promise<CTIRes> {
  const id = random64Hex()
  const response = await fetch(baseUrl + url, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      'X-B3-TraceId': id,
      'X-B3-SpanId': id
    },
    body: JSON.stringify(data)
  })

  if (!response.ok) {
    apiLogger.error(
      `api response | ${url} | Request failed with status ${response.status}`
    )
  }

  return response.json() as Promise<CTIRes>
}

export const hsTrackJPOST = ({
  baseUrl,
  url = '',
  data
}: Request): Promise<CTIRes> => {
  // eslint-disable-next-line no-async-promise-executor
  return new Promise(async (resolve, reject) => {
    try {
      const res = await JPOST({ baseUrl, url, data })
      const { code, msg } = res
      console.log(msg)
      if (code === 0) {
        apiLogger.log(`api response | ${url} | ${JSON.stringify(res)}`)
      } else {
        apiLogger.error(`api response | ${url} | ${JSON.stringify(res)}`)
      }
      resolve(res)
    } catch (e) {
      apiLogger.error(`api response | ${url} | ${JSON.stringify(e)}`)
      reject(e)
    }
  })
}
